@@ -94,7 +94,6 @@ Bootstrapper.prototype.readKeys = function (callback) {
   } else if (this.keys.hasOwnProperty('public') && this.keys.hasOwnProperty('private')) {
     iterateKeys([this.keys.public, this.keys.private]);
   }
-
 };
 
 //
@@ -219,7 +218,15 @@ Bootstrapper.prototype.createServer = function (options) {
           self.bootstrapServer(options)
             .on('error', onError)
             .on('complete', function () {
-              emitter.emit('complete', server);
+              if (!options.afterActive) {
+                return emitter.emit('complete', server);
+              }
+
+              options.afterActive(self.compute, server, function (err) {
+                return err
+                  ? emitter.emit('error', err)
+                  : emitter.emit('complete', server);
+              });
             });
         });
       }
@@ -300,13 +307,13 @@ Bootstrapper.prototype.bootstrapServer = function (options) {
   // Now that the server is active upload any required files
   // and run the specified `options.commands`.
   //
-  async.series([
+  async.waterfall([
     //
     // 1. Create any remote directories for files that will
     //    be uploaded.
     //
     function createDirs(next) {
-      if (!options.files) {
+      if (!options.files || !options.files.length) {
         return next();
       }
 
@@ -334,7 +341,7 @@ Bootstrapper.prototype.bootstrapServer = function (options) {
     // 2. Upload any required files.
     //
     function uploadFiles(next) {
-      return options.files
+      return options.files && options.files.length
         ? async.forEachSeries(options.files, uploadFile, next)
         : next();
     },
@@ -342,7 +349,7 @@ Bootstrapper.prototype.bootstrapServer = function (options) {
     // 3. Bootstrap the server with the appropriate commands.
     //
     function bootstrap(next) {
-      if (!options.commands) {
+      if (!options.commands || !options.commands.length) {
         return next();
       }
 
@@ -359,20 +366,20 @@ Bootstrapper.prototype.bootstrapServer = function (options) {
           hasErr = true;
           next(err);
         }
-      }).on('complete', function () {
+      }).on('complete', function (server, stdout) {
         if (!hasErr) {
-          next();
+          next(null, stdout);
         }
       });
     }
-  ], function (err) {
+  ], function (err, stdout) {
     //
     // 3. Emit the approrpriate event depending if there is
     //    an error or not.
     //
     return err
       ? onError(err)
-      : emitter.emit('complete', options.server);
+      : emitter.emit('complete', options.server, stdout);
   });
 
   return emitter;
@@ -775,11 +782,10 @@ Bootstrapper.prototype._exec = function (command, options) {
 // active, ready to accept commands.
 //
 Bootstrapper.prototype._sshPoll = function (options, callback) {
-  // TODO (mmalecki): make maxTries and interval configurable
-  var self = this,
-      maxTries = 10,
-      interval = 30 * 1000,
-      tries = 0;
+  var interval   = options.pollInterval || 30 * 1000,
+      maxRetries = options.pollMax      || 10,
+      attempts   = 0
+      self = this;
 
   function poll() {
     var child = self.ssh({
@@ -791,8 +797,8 @@ Bootstrapper.prototype._sshPoll = function (options, callback) {
     });
 
     child.on('error', function () {
-      ++tries;
-      if (tries === maxTries) {
+      ++attempts;
+      if (++attempts >= maxRetries) {
         return callback(new Error('Server didn\'t become active in timely fashion'));
       }
 
